@@ -40,12 +40,14 @@ struct SalsaState {
 }
 
 #[derive(Clone)]
-pub struct Salsa20 {
+pub struct Salsa<const ROUNDS: usize> {
     state: SalsaState,
     output: [u8; 64],
     offset: usize,
 }
 
+pub type Salsa20 = Salsa<20>;
+
 const S7: u32x4 = u32x4(7, 7, 7, 7);
 const S9: u32x4 = u32x4(9, 9, 9, 9);
 const S13: u32x4 = u32x4(13, 13, 13, 13);
@@ -97,12 +99,12 @@ fn rowround(state: &mut SalsaState) {
     add_rotate_xor!(state.d, state.a, state.b, S18);
 }
 
-impl Salsa20 {
+impl<const ROUNDS: usize> Salsa<ROUNDS> {
     pub fn new(key: &[u8], nonce: &[u8]) -> Salsa20 {
         assert!(key.len() == 16 || key.len() == 32);
         assert!(nonce.len() == 8);
-        Salsa20 {
-            state: Salsa20::expand(key, nonce),
+        Salsa {
+            state: Salsa::<ROUNDS>::expand(key, nonce),
             output: [0; 64],
             offset: 64,
         }
@@ -111,17 +113,17 @@ impl Salsa20 {
     pub fn new_xsalsa20(key: &[u8], nonce: &[u8]) -> Salsa20 {
         assert!(key.len() == 32);
         assert!(nonce.len() == 24);
-        let mut xsalsa20 = Salsa20 {
-            state: Salsa20::expand(key, &nonce[0..16]),
+        let mut xsalsa = Salsa {
+            state: Salsa::<ROUNDS>::expand(key, &nonce[0..16]),
             output: [0; 64],
             offset: 64,
         };
 
         let mut new_key = [0; 32];
-        xsalsa20.hsalsa20_hash(&mut new_key);
-        xsalsa20.state = Salsa20::expand(&new_key, &nonce[16..24]);
+        xsalsa.hsalsa_hash(&mut new_key);
+        xsalsa.state = Salsa20::expand(&new_key, &nonce[16..24]);
 
-        xsalsa20
+        xsalsa
     }
 
     fn expand(key: &[u8], nonce: &[u8]) -> SalsaState {
@@ -205,7 +207,7 @@ impl Salsa20 {
         self.offset = 0;
     }
 
-    fn hsalsa20_hash(&mut self, out: &mut [u8]) {
+    fn hsalsa_hash(&mut self, out: &mut [u8]) {
         let mut state = self.state.clone();
         for _ in 0..10 {
             columnround(&mut state);
@@ -251,22 +253,25 @@ impl Salsa20 {
     }
 }
 
-pub fn hsalsa20(key: &[u8], nonce: &[u8], out: &mut [u8]) {
+pub fn hsalsa<const ROUNDS: usize>(key: &[u8], nonce: &[u8], out: &mut [u8]) {
     assert!(key.len() == 32);
     assert!(nonce.len() == 16);
+    assert!(ROUNDS == 8 || ROUNDS == 12 || ROUNDS == 20);
+
     let mut h = Salsa20 {
         state: Salsa20::expand(key, nonce),
         output: [0; 64],
         offset: 64,
     };
-    h.hsalsa20_hash(out);
+    h.hsalsa_hash(out);
+}
+
+pub fn hsalsa20(key: &[u8], nonce: &[u8], out: &mut [u8]) {
+    hsalsa::<20>(key, nonce, out)
 }
 
 #[cfg(test)]
 mod test {
-    use alloc::vec::Vec;
-    use std::iter::repeat;
-
     use super::Salsa20;
 
     use crate::digest::Digest;
@@ -321,15 +326,20 @@ mod test {
             0x66, 0x25, 0x6c, 0xe4,
         ];
         let nonce = [0x82, 0x19, 0xe0, 0x03, 0x6b, 0x7a, 0x0b, 0x37];
-        let input: Vec<u8> = repeat(0).take(4194304).collect();
-        let mut stream: Vec<u8> = repeat(0).take(input.len()).collect();
         let output_str = "662b9d0e3463029156069b12f918691a98f7dfb2ca0393c96bbfc6b1fbd630a2";
 
         let mut salsa20 = Salsa20::new(&key, &nonce);
-        salsa20.process(input.as_ref(), &mut stream);
 
+        // hash 4194304 0-bytes (512*8192) in SHA256
         let mut sh = Sha256::new();
-        sh.input(stream.as_ref());
+
+        let block = [0u8; 512];
+        let mut stream_output = [0u8; 512];
+        for _ in 0..8192 {
+            salsa20.process(&block, &mut stream_output);
+            sh.input(&stream_output);
+        }
+
         let out_str = sh.result_str();
         assert!(&out_str[..] == output_str);
     }
@@ -369,7 +379,6 @@ mod test {
 #[cfg(all(test, feature = "with-bench"))]
 mod bench {
     use super::Salsa20;
-    use crate::symmetriccipher::SynchronousStreamCipher;
     use test::Bencher;
 
     #[bench]
